import { get as getStoreData } from 'svelte/store';
export default class Pages {
    pageNumber;
    rowCount;
    rowsPerPage;
    triggerChange;
    pages;
    paginationType;
    triggeredPaginate;
    constructor(context) {
        this.paginationType = context.pagination.type;
        if (this.paginationType === 'cursor') {
            this.triggeredPaginate = context.triggeredPaginate;
        }
        else {
            this.pageNumber = context.pageNumber;
            this.rowCount = context.rowCount;
            this.pages = context.pages;
        }
        this.rowsPerPage = context.rowsPerPage;
        this.triggerChange = context.triggerChange;
    }
    get() {
        return this.pages;
    }
    goTo(number) {
        this.pageNumber.update(store => {
            const $rowsPerPage = this.getRowsPerPage();
            if ($rowsPerPage) {
                const $rowsTotal = this.getTotalRowCount();
                if (number >= 1 && number <= Math.ceil($rowsTotal / $rowsPerPage)) {
                    store = number;
                    this.triggerChange.update(store => { return store + 1; });
                }
            }
            return store;
        });
    }
    previous() {
        if (this.paginationType === 'cursor') {
            this.triggeredPaginate.set("before");
            this.triggerChange.update(store => { return store + 1; });
        }
        else {
            const number = this.getPageNumber() - 1;
            this.goTo(number);
        }
    }
    next() {
        if (this.paginationType === 'cursor') {
            this.triggeredPaginate.set("after");
            this.triggerChange.update(store => { return store + 1; });
        }
        else {
            const number = this.getPageNumber() + 1;
            this.goTo(number);
        }
    }
    getPageNumber() {
        return getStoreData(this.pageNumber);
    }
    getTotalRowCount() {
        return getStoreData(this.rowCount).total;
    }
    getRowsPerPage() {
        return getStoreData(this.rowsPerPage);
    }
}
